#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QShortcut>
#include <QDialog>
#include <QDialogButtonBox>
#include <QFormLayout>
#include <QLineEdit>
#include <QFile>
#include <QTextStream>
#include <QString>
#include <QCoreApplication>
#include <QSqlDatabase>
#include <QMouseEvent>
#include <iostream>
#include <QSqlQuery>
#include <QSqlError>
#include <QDebug>
#include <QMessageBox>
#include <QSqlRecord>
#include <QSqlTableModel>
#include <QList>
#include <QSqlDatabase>
#include <iomanip>
#include <windows.h>

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{


    ui->setupUi(this);
    ui->lineEdit->setPlaceholderText("Enter the command");
    ui->lineEdit->setDisabled(true);
    ui->lineEdit->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
    ui->textEdit_log->setFontPointSize(9);
    ui->textEdit_log->setReadOnly(true);
    ui->textEdit_log->setPlaceholderText("Primarily connect to Data Base");
    ui->textEdit_log->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);

    connect(ui->pushButton, SIGNAL(clicked()), this, SLOT(ok_clicked()));
    connect(ui->pushButton_2, SIGNAL(clicked()), this, SLOT(clear_clicked()));
    connect(ui->listWidget, &QListWidget::itemClicked, this, [this](QListWidgetItem* item)
    {
        DBUpdate(item->text());
    });
    connect(ui->actionExit, SIGNAL(triggered()), this, SLOT(on_actionExit_triggered()));

    QShortcut *shortcut = new QShortcut(Qt::Key_Return, this);
    QObject::connect(shortcut, SIGNAL(activated()), this, SLOT(ok_clicked()));

}

void MainWindow::NewLogString(const QString& command) {
     ui->textEdit_log->append(command);
}

void MainWindow::ok_clicked() {
    QString command = ui->lineEdit->text();
    QSqlQuery query;

    bool cmd = query.exec(command);

    if (command != nullptr) {

        std::string str = command.toStdString();
        if (str.substr(0, 6) == "SELECT" || str.substr(0, 4) == "WITH") {

            QSqlQueryModel *model_q = new QSqlQueryModel(ui->tableView);
            model_q->setQuery(command);
            ui->tableView->setModel(model_q);
        }

        ui->textEdit_log->append(command);
        ui->textEdit_log->append("\n");

        if (!cmd) {
              ui->textEdit_log->append(query.lastError().text());
        }
        else if (!(str.substr(0, 6) == "SELECT" || str.substr(0, 4) == "WITH")){
            ui->textEdit_log->append("Command execution successful!");
            DBUpdate("");
        }
    }
}



void MainWindow::clear_clicked() {
    ui->lineEdit->clear();
}

MainWindow::~MainWindow()
{
    delete ui;
}


void MainWindow::on_actionExit_triggered()
{
      CloseDB();
      QApplication::quit();
}

void MainWindow::closeEvent(QCloseEvent *event) {
    QMessageBox::StandardButton ret;
    ret = QMessageBox::question( this,  QApplication::applicationName(), tr(" Do you want to close programm ? "),
                                 QMessageBox::Yes | QMessageBox::No , QMessageBox::No );
    if (ret == QMessageBox::No)
        event->ignore();
}

void MainWindow::on_actionConnect_triggered()
{
    QDialog dlg(this);
    dlg.setWindowTitle(tr("Connect configuration"));
    dlg.setWindowIcon(QIcon("D:\\Programms\\Qt Labs\\manager_db\\Manager_DB_\\logo_connection.png"));
    dlg.isModal(); //делаем приоритетным окном
    dlg.setWindowFlags (dlg.windowFlags() & ~Qt::WindowContextHelpButtonHint); //убираем знак вопроса
    QLineEdit *SqlDatabase = new QLineEdit(&dlg);
    QLineEdit *HostName = new QLineEdit(&dlg);
    QLineEdit *DatabaseName = new QLineEdit(&dlg);
    QLineEdit *UserName = new QLineEdit(&dlg);
    QLineEdit *Password = new QLineEdit(&dlg);
    QLineEdit *Port = new QLineEdit(&dlg);

    QDialogButtonBox *btn_box = new QDialogButtonBox(&dlg);
    btn_box->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);

    connect(btn_box, &QDialogButtonBox::accepted, &dlg, &QDialog::accept);
    connect(btn_box, &QDialogButtonBox::rejected, &dlg, &QDialog::reject);

    QFormLayout *layout = new QFormLayout();
    layout->addRow(tr("SqlDatabase:"), SqlDatabase);
    layout->addRow(tr("HostName:"), HostName);
    layout->addRow(tr("DatabaseName:"), DatabaseName);
    layout->addRow(tr("UserName:"), UserName);
    layout->addRow(tr("Password:"), Password);
    layout->addRow(tr("Port:"), Port);
    layout->addWidget(btn_box);

    //Пошла работа с файлами
    QFile file("config.txt");

    if (file.open(QIODevice::ReadOnly)) {
        QTextStream in(&file);
        SqlDatabase->setText(config.SqlDatabase = in.readLine());
        HostName->setText(config.HostName = in.readLine());
        DatabaseName->setText(config.DatabaseName = in.readLine());
        UserName->setText(config.UserName = in.readLine());
        Password->setText(config.Password = in.readLine());
        Port->setText(config.Port = in.readLine());
        file.close();
    }

    dlg.setLayout(layout);

    if(dlg.exec() == QDialog::Accepted) {

        if(file.open(QIODevice::WriteOnly | QIODevice::Text))
            {
                QTextStream out(&file);
                out << SqlDatabase->text() << "\n";
                config.SqlDatabase = SqlDatabase->text();
                out << HostName->text() << "\n";
                config.HostName = HostName->text();
                out << DatabaseName->text() << "\n";
                config.DatabaseName = DatabaseName->text();
                out << UserName->text() << "\n";
                config.UserName = UserName->text();
                out << Password->text() << "\n";
                config.Password = Password->text();
                out << Port->text() << "\n";
                config.Port = Port->text();
                file.close();
        }
        ConnectToServer();
    }
}

void MainWindow::ConnectToServer() {
    myDB = QSqlDatabase::addDatabase(config.SqlDatabase);
    myDB.setHostName(config.HostName);
    myDB.setDatabaseName(config.DatabaseName);
    myDB.setUserName(config.UserName);
    myDB.setPassword(config.Password);
    myDB.setPort(config.Port.toInt());

    if (!myDB.open()) {
        NewLogString("DB is not open!");
        NewLogString("ERR type:  ");
        NewLogString(myDB.lastError().text());
    }
    else {
        NewLogString("DB is open!");
        ui->lineEdit->setDisabled(false);
    }
    DBUpdate("");
}


void MainWindow::DBUpdate(const QString table_name) {

    QSqlTableModel *model = new QSqlTableModel(this);

    QStringList table_list = myDB.tables();
    ui->listWidget->clear();
    ui->listWidget->addItems(table_list);

    QStringList headers;

    if (table_name.size()) {
         model->setTable(table_name);
    }
    else if (table_list.size()) {
        model->setTable(table_list[0]);
    }

    for (int i = 0; i < model->columnCount(); i++)
    {
        headers.append(model->headerData(i, Qt::Horizontal).toString());
    }

    for (int i = 0, j = 0; i < model->columnCount(); i++, j++)
    {
        model->setHeaderData(i, Qt::Horizontal, headers[j]);
    }


    ui->tableView->setModel(model);
    model->select();
}

void MainWindow::CloseDB() {

    myDB.close();
}

